/**
 * Created by FreetimeCoder 2012
 * www.freetimestudio.de
 * Use as you wish, credits are appreciated ;)
 */
class SeqAct_QTEButton extends SeqAct_Latent;

//How long this QTE is
var(QuickTimeEvent) float fDuration;
//How long this QTE will go in
var                 float fRemainingTime;
//If the QTE is paused
var                 bool  bPaused;
//If the QTE has been activated and not yet finished
var                 bool  bQTActive;

//Value which is modified
var                 float fValue;

//Reaching max value finishes the QTE, even if the time has not run out, yet
var(QuickTimeEvent) bool bReachingValueFinishes;

//fValue at the start of the QTE
var(QuickTimeEvent) float fStartValue;
//Number that Value has to be to beat the QTE
var(QuickTimeEvent) float fEndValue;
//Step size to increment the value each time the correct button is hit
var(QuickTimeEvent) float fValueStep;
//Decrease the Value over time
var(QuickTimeEvent) float fValueDecreasePerSec;

/**
 * Reset all values to start
 */
function Reset()
{
	fValue = fStartValue;
	fRemainingTime = fDuration;
	bAborted=false;
	bQTActive=true;

	OutputLinks[0].bHasImpulse = false;
	OutputLinks[1].bHasImpulse = false;
	OutputLinks[2].bHasImpulse = false;
	OutputLinks[3].bHasImpulse = false;
}

/**
 * QTE succeeded, activate output and finish
 */
function Success()
{
	ActivateOutputLink(2);
	Finish();
}

/**
 * QTE failed, activate output and finish
 */
function Failure()
{
	ActivateOutputLink(3);
	Finish();
}

/**
 * Activate finished output and deactivate
 */
function Finish()
{
	bQTActive=false;
	ActivateOutputLink(0);
}

/**
 * Activate aborted output and deactivate
 */
function Abort()
{
	bAborted = true;
	bQTActive=false;
	ActivateOutputLink(1);
}

/**
 * Update Value output variable and activate "Value Changed" output
 */
function UpdatePercent()
{
	local int i,j;
	OutputLinks[4].bHasImpulse = false;

	//TODO: Find a better way to set values
	//Iterate through all variable links to set the correct value
	for(i=0;i<VariableLinks.Length;i++)
	{
		if(VariableLinks[i].LinkDesc=="Percent")
		{
			for(j=0;j<VariableLinks[i].LinkedVariables.Length;j++)
			{
				if(SeqVar_Float(VariableLinks[i].LinkedVariables[j])!=none)
				{
					SeqVar_Float(VariableLinks[i].LinkedVariables[j]).FloatValue = fValue/fEndValue;
				}
			}
		}
	}

	ActivateOutputLink(4);
}

/**
 * Update latent action
 */
event bool Update(float DeltaTime)
{

	if(InputLinks[0].bHasImpulse)   //Start
	{
		//If action was activated and not pause, start by resetting
		if(!bPaused)
		{
			Reset();
		}
		//Set paused to false, in case it was set to true
		bPaused=false;
	}


	//Only update everything else in case the QTE is active
	if(bQTActive)
	{
		if(!bPaused)
		{
			//Update time and check if value target was reached
			fRemainingTime -= DeltaTime;
			if (fDuration!=0 && fRemainingTime<=0)
			{
				if(fValue>=fEndValue)
				{
					Success();
				}
				else
				{
					//Fails if time runs out without reaching a high enough value
					Failure();
				}

				//quit latent execution
				return false;
			}

			//Drecrease value 
			if(fValueDecreasePerSec!=0)
			{
				fValue = FMax(0, fValue-fValueDecreasePerSec*DeltaTime);
				UpdatePercent();
			}

			//Right key was pressed, increase value
			if(InputLinks[3].bHasImpulse)
			{
				fValue += fValueStep;
				UpdatePercent();
				
				if(fValue>=fEndValue)
				{
					if(bReachingValueFinishes)
					{
						Success();
						//quit latent execution
						return false;
					}
				}

			}
		}

		//Stop
		if(InputLinks[1].bHasImpulse)  
		{
			Abort();
			return false;
		}

		//Pause
		if(InputLinks[2].bHasImpulse)  
		{
			bPaused = true;
		}

		//Wrong Key
		if(InputLinks[4].bHasImpulse)  
		{
			Failure();
			return false;
		}
	}

	//Continue executing latent action
    return true;
}

DefaultProperties
{
	ObjName = "QTE Button Press"
	ObjCategory = "Quick Time Events"

	InputLinks.Empty()
	InputLinks(0)=(LinkDesc="Start")
	InputLinks(1)=(LinkDesc="Stop")
	InputLinks(2)=(LinkDesc="Pause")
	InputLinks(3)=(LinkDesc="Right Key")
	InputLinks(4)=(LinkDesc="Wrong Key")

	OutputLinks(2)=(LinkDesc="Success")
	OutputLinks(3)=(LinkDesc="Failure")
	OutputLinks(4)=(LinkDesc="Value Changed")

	//TODO: Target is not used, but it may be useful in case you want to have multiple players
	VariableLinks(0)=(bHidden=true)
	VariableLinks(1)=(ExpectedType=class'SeqVar_Float',LinkDesc="fDuration",MaxVars=1,bClampedMax=true,PropertyName=fDuration)
	VariableLinks(2)=(ExpectedType=class'SeqVar_Float',LinkDesc="Percent",  MaxVars=1,bClampedMax=true,bWriteable=true,bSequenceNeverReadsOnlyWritesToThisVar=true)

	bAutoActivateOutputLinks=false

	fDuration = 2

	fValue = 0
	fStartValue = 0
	fEndValue = 1
	fValueStep = 1
	fValueDecreasePerSec = 0

	bReachingValueFinishes=true
}