/**
 * Created by FreetimeCoder 2012
 * www.freetimestudio.de
 * Use as you wish, credits are appreciated ;)
 */
class SeqAct_QTEStickRotation extends SeqAct_QTEButton;

//Restrict the maximum angle delta to prevent wild flinging of the analog stick
var(QuickTimeEvent) float   fMaxAngleStep;
//Invert the value to make the player rotate the stick counter clockwise
var(QuickTimeEvent) bool    bCounterClockwise;
//Positive and negative delta angles are accepted
var(QuickTimeEvent) bool    bTwoDirectional;

//Last stick direction to calculate delta angle
var                 Vector  vOldDirection;
//current direction of stick
var                 Vector  vDirection;

/**
 * Reset all values to start
 */
function Reset()
{
	super.Reset();

	vOldDirection = vDirection;
}

/**
 * Update vDirection depending on vector input
 */
function UpdateDirection()
{
	local int i;

	//TODO: Find a better way to get values
	//Iterate through all variable links to get the correct value
	for(i=0;i<VariableLinks.Length;i++)
	{
		if(VariableLinks[i].LinkDesc=="Input Vector")
		{
			if(SeqVar_Vector(VariableLinks[i].LinkedVariables[0])!=none)
			{
				vDirection = SeqVar_Vector(VariableLinks[i].LinkedVariables[0]).VectValue;
			}
		}
	}
}

/**
 * Update latent action
 */
event bool Update(float DeltaTime)
{
	local float fOldAngle, fAngle;

	//Start
	if(InputLinks[0].bHasImpulse)   
	{
		if(!bPaused)
		{
			Reset();
		}

		bPaused=false;
	}

	//Only update everything else in case the QTE is active
	if(bQTActive)
	{
		if(!bPaused)
		{
			//Update time and check if value target was reached
			fRemainingTime -= DeltaTime;
			if(fDuration!=0 && fRemainingTime<=0)
			{
				if(fValue>=fEndValue)
				{
					Success();
				}
				else
				{
					//Fails if time runs out without reaching a high enough value
					Failure();
				}

				//quit latent execution
				return false;
			}

			//Drecrease value 
			if(fValueDecreasePerSec!=0)
			{
				fValue = FMax(0, fValue-fValueDecreasePerSec*DeltaTime);
				UpdatePercent();
			}
		}

		//Stop
		if(InputLinks[1].bHasImpulse)  
		{
			Abort();
			return false;
		}
	
		//Pause
		if(InputLinks[2].bHasImpulse)  
		{
			bPaused = true;
		}

		//Input vector has changed, update angle values
		if(InputLinks[3].bHasImpulse && !bPaused)  
		{
			//Read new vector
			UpdateDirection(); 

			//Calculate angle between new and last direction
			fOldAngle = Atan2(vOldDirection.X,vOldDirection.Y);
			fAngle = (Atan2(vDirection.X,vDirection.Y) - fOldAngle)*RadToDeg;

			//Only change value in case the angle has changed and is below max angle setp
			if(fAngle!=0 && Abs(fAngle)<=fMaxAngleStep)
			{
				//Inverse delta angle if rotation should be counter clockwise
				if(bCounterClockwise)
				{
					fAngle *= -1;
				}

				//Only increase value in the right direction
				if(fAngle<0)
				{
					if(!bTwoDirectional)
					{
						fAngle = 0;
					}
				}

				//Clamp value
				fValue = FMax(0,fValue+fAngle*fValueStep);

				//Update output value
				UpdatePercent();
			}

			//Store vector for next time
			vOldDirection = vDirection;

			//Check for end
			if(fValue>=fEndValue)
			{
				if(bReachingValueFinishes)
				{
					Success();
					return false;
				}
			}
		}
	}

	//Continue latent execution
    return true;
}


DefaultProperties
{
	ObjName = "QTE Stick Rotation"

	VariableLinks(3)=(ExpectedType=class'SeqVar_Vector',LinkDesc="Input Vector",MaxVars=1,bClampedMax=true, PropertyName=vDirection)

	InputLinks.Empty()
	InputLinks(0)=(LinkDesc="Start");
	InputLinks(1)=(LinkDesc="Stop");
	InputLinks(2)=(LinkDesc="Pause");
	InputLinks(3)=(LinkDesc="Update Input");

	bTwoDirectional = false

	fValue = 0
	fStartValue = 0
	fEndValue = 360
	fValueStep = 1
	fMaxAngleStep = 30;
}
